/**
 * Copyright (c) 2014 - 2021, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup ble_sdk_app_template_main main.c
 * @{
 * @ingroup ble_sdk_app_template
 * @brief Template project main file.
 *
 * This file contains a template for creating a new application. It has the code necessary to wakeup
 * from button, advertise, get a connection restart advertising on disconnect and if no new
 * connection created go back to system-off mode.
 * It can easily be used as a starting point for creating a new application, the comments identified
 * with 'YOUR_JOB' indicates where and how you can customize.
 */

#include <stdbool.h>
#include <stdint.h>
#include <string.h>

#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "app_timer.h"
#include "fds.h"
#include "peer_manager.h"
#include "peer_manager_handler.h"
#include "bsp_btn_ble.h"
#include "sensorsim.h"
#include "ble_conn_state.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "nrf_pwr_mgmt.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_drv_twi.h"
#include "sht4x.h"
#include "lis3dh.h"

const uint8_t device_name_arr[2]        = {'N', 2};
#define DEVICE_NAME                     device_name_arr//"Nordic_Template"      /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME               "NordicSemiconductor"                   /**< Manufacturer. Will be passed to Device Information Service. */
#define APP_ADV_INTERVAL                32 //80 = 50ms //300 karna hae                  /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */

#define APP_ADV_DURATION                60 //200 = 2s//1800 50, 300 ,20 karna hae                 /**< The advertising duration (180 seconds) in units of 10 milliseconds. */
#define APP_BLE_OBSERVER_PRIO           3                                       /**< Application's BLE observer priority. You shouldn't need to modify this value. */
#define APP_BLE_CONN_CFG_TAG            1                                       /**< A tag identifying the SoftDevice BLE configuration. */

#define ADV_INTERVAL                    APP_TIMER_TICKS(1000*30)                /**< Interval after which device will send notification (in milli seconds). */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(100, UNIT_1_25_MS)        /**< Minimum acceptable connection interval (0.1 seconds). */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(200, UNIT_1_25_MS)        /**< Maximum acceptable connection interval (0.2 second). */
#define SLAVE_LATENCY                   0                                       /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)         /**< Connection supervisory timeout (4 seconds). */

#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                   /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                  /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                       /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_BOND                  1                                       /**< Perform bonding. */
#define SEC_PARAM_MITM                  0                                       /**< Man In The Middle protection not required. */
#define SEC_PARAM_LESC                  0                                       /**< LE Secure Connections not enabled. */
#define SEC_PARAM_KEYPRESS              0                                       /**< Keypress notifications not enabled. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE                    /**< No I/O capabilities. */
#define SEC_PARAM_OOB                   0                                       /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                       /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                      /**< Maximum encryption key size. */

#define DEAD_BEEF                       0xDEADBEEF                              /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */
//#define TEMP_VDD                        28

////-------------
#define BT0_LONG_PUSH  BSP_EVENT_KEY_0		

NRF_BLE_GATT_DEF(m_gatt);                                                       /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                         /**< Context for the Queued Write module.*/
BLE_ADVERTISING_DEF(m_advertising);                                             /**< Advertising module instance. */

//APP_TIMER_DEF(m_adv_timer_id);                                         /**< timer to send notification. */

#define TOP_INDEX                       3
APP_TIMER_DEF(m_repeated_timer_id); 
APP_TIMER_DEF(motion_sensor_timer_id); 
APP_TIMER_DEF(sensor_timer_id); 
uint8_t battery=100;
uint8_t data_to_advertise[27]; 
//SHT40 variable
float temp;
float hum;
int16_t temp1,hum1;
int flag=0;                    // variable to count two readings
uint8_t flag1=0;
uint8_t remaining_packet=152;
extern int acc_working_condition;
uint8_t temp_working_condition=0;
int16_t tmp_int16[3] = {0};
static int Motion_index=0;

// button pressed for 5 sec var
static uint8_t sleepFlag = 1;
static uint32_t button_tick = 0;



#define ADV_DATA_UPDATE_INTERVAL        APP_TIMER_TICKS(2000)                              //5000 karna hae  - 120000       500
#define ADV_DATA_UPDATE                 APP_TIMER_TICKS (10000)//(30000)                                  //60000*4           
#define Motion_Data_Update              APP_TIMER_TICKS(100)
static const nrf_drv_twi_t i2c1_instance = NRF_DRV_TWI_INSTANCE(0);
static const nrf_drv_twi_t  m_twi = NRF_DRV_TWI_INSTANCE(1);
#define PAYLOAD_SIZE                    1

#define APP_COMPANY_IDENTIFIER          0x0059                                   /**< Company identifier for Nordic Semiconductor ASA. as per www.bluetooth.org. */

typedef struct
{
uint8_t x_msb;
uint8_t x_lsb;	
uint8_t y_msb;
uint8_t y_lsb;	
uint8_t z_msb;
uint8_t z_lsb;	
}motion_data;
motion_data Motion_data[600];                // structure to hold Lis3dh data 8
uint8_t*ptr;

typedef struct
{
	uint8_t manufacturing_data_payload_list[27];
}manuf_payload_t;
manuf_payload_t manuf_payload_arr[3] = {
		//012345678901234567890123456
        {"1DataPacketTestingString123"},
		{"2DataPacketTestingString567"},
		{"3DataPacketTestingString9AB"},
};

static uint16_t m_conn_handle = BLE_CONN_HANDLE_INVALID;                        /**< Handle of the current connection. */

/* YOUR_JOB: Declare all services structure your application is using
 *  BLE_XYZ_DEF(m_xyz);
 */

// YOUR_JOB: Use UUIDs for service(s) used in your application.
//static ble_uuid_t m_adv_uuids[] =                                               /**< Universally unique service identifiers. */
//{
//    {BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE}
//};

static void advertising_init();
static void advertising_start();
//static void adv_timeout_handler(void * p_context);
void send_notification(void);

/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num   Line number of the failing ASSERT call.
 * @param[in] file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/**@brief Function for handling Peer Manager events.
 *
 * @param[in] p_evt  Peer Manager event.
 */
static void pm_evt_handler(pm_evt_t const * p_evt)
{
    pm_handler_on_pm_evt(p_evt);
    pm_handler_disconnect_on_sec_failure(p_evt);
    pm_handler_flash_clean(p_evt);

    switch (p_evt->evt_id)
    {
        case PM_EVT_PEERS_DELETE_SUCCEEDED:
            advertising_start(false);
            break;

        default:
            break;
    }
}

static void packet_to_advertise()
{  
	    NRF_LOG_INFO("enter into packet formation");
	    int i,j;	
	    
     if (flag1<=2)
     {
			 NRF_LOG_INFO("temp and battery packet ");
			 --remaining_packet;
			 data_to_advertise[0]=remaining_packet;
			 data_to_advertise[1]=1;
			 data_to_advertise[2]=temp_working_condition;
			 data_to_advertise[3]=(uint8_t)((temp1 & 0xFF00) >> 8);              //MSB of data
		  data_to_advertise[4]=(uint8_t)(temp1 & 0xFF) ;                     //LSB  of data
			data_to_advertise[5]=battery;  

     }	
		 else
		 {
			 --remaining_packet;
	     data_to_advertise[0]=remaining_packet;
			 data_to_advertise[1]=2;
			 data_to_advertise[2]=acc_working_condition; 			
			ptr=&Motion_data[Motion_index].x_msb;	
			for(i=3;i<27;i++)
			{
				  
			 data_to_advertise[i]=*(ptr+(i-3));
								
			}
	  	Motion_index=Motion_index+4;
			
			NRF_LOG_INFO("value of motion index %d",Motion_index);
		
		}
      
    	
			for(j=0;j<27;j++)
			{
		    NRF_LOG_INFO("data to advertise %d",data_to_advertise[j]);
			}		
		NRF_LOG_INFO("come out of the packet formation");
		
}  



static void repeated_timer_handler(void * p_context)
{
  
    // packet formation	
	  ++flag1;
    packet_to_advertise();

	  advertising_init();	                              
    NRF_LOG_INFO("Advertising start");
	  NRF_LOG_INFO("value of flag1 %d",flag1);
    ret_code_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
    APP_ERROR_CHECK(err_code);
	
	  	if(flag1==152)
		{
	  	Motion_index=0;
  		flag1=0;
  	  remaining_packet=152;
      app_timer_stop(m_repeated_timer_id);			
		  ret_code_t error_code;
      error_code = app_timer_start(sensor_timer_id,                                         
                                    ADV_DATA_UPDATE, 
                                    NULL);
      APP_ERROR_CHECK(error_code);
      acc_working_condition=0;
	  	temp_working_condition=0;	
			
		 }

     
	 
 	
}




static void motion_sensor_timer_handler(void *p_context)
{
    float LIS3DH_Acc[3];
	  twi_init(); 
	  LIS3DH_Init();                                  //LIS3DH Sensor Init
	  LIS3DH_GetAccData(LIS3DH_Acc);
    int16_t tmp_int16[3] = {0};
 
     tmp_int16[0] =(int16_t)( LIS3DH_Acc[0]);
		 tmp_int16[1] =(int16_t)( LIS3DH_Acc[1]);
		 tmp_int16[2] =(int16_t)( LIS3DH_Acc[2]);
		 
		 NRF_LOG_INFO(" value of temp x %d ",tmp_int16[0]);
		 NRF_LOG_INFO(" value of temp y %d ",tmp_int16[1]);
		 NRF_LOG_INFO(" value of temp z %d ",tmp_int16[2]);
		
	  	Motion_data[flag].x_msb= (uint8_t)((tmp_int16[0] & 0xFF00)>>8);
		  Motion_data[flag].x_lsb= (int8_t)(tmp_int16[0] & 0xFF);
	  	Motion_data[flag].y_msb= (int8_t)((tmp_int16[1] & 0xFF00)>>8);
		  Motion_data[flag].y_lsb= (int8_t)(tmp_int16[1] & 0xFF);
	  	Motion_data[flag].z_msb= (int8_t)((tmp_int16[2] & 0xFF00)>>8);
	  	Motion_data[flag].z_lsb= (int8_t)(tmp_int16[2] & 0xFF);
	//	 NRF_LOG_INFO(" value of x msb %d ",Motion_data[flag].x_msb);
	//	 NRF_LOG_INFO(" value of x lsb %d ",Motion_data[flag].x_lsb);
	//	 NRF_LOG_INFO(" value of y msb %d ",Motion_data[flag].y_msb);
	//	 NRF_LOG_INFO(" value of y lsb %d ",Motion_data[flag].y_lsb);
  //   NRF_LOG_INFO(" value of z msb %d ",Motion_data[flag].z_msb);
	//	 NRF_LOG_INFO(" value of z lsb %d ",Motion_data[flag].z_lsb);
	   NRF_LOG_INFO(" value of flag %d ",flag);
	 	 ++flag;
				
	   nrf_drv_twi_disable(&m_twi);          // disable twi
	   nrf_drv_twi_uninit(&m_twi);  
	 
	   if(flag==600)
	   {
	     flag=0;		         
		   app_timer_stop(motion_sensor_timer_id);				
		   ret_code_t error_code;
       error_code = app_timer_start(m_repeated_timer_id, 
                                    ADV_DATA_UPDATE_INTERVAL, 
                                    NULL);
       APP_ERROR_CHECK(error_code);	   
		
    	}
		 
			  	
	
}




static void sensor_timer_handler(void *p_context)
{
 
	     UNUSED_PARAMETER(p_context);
	   
     	 sensirion_i2c_init();
			 while (sht4x_probe() != STATUS_OK) 
				{
						NRF_LOG_INFO("SHT sensor probing failed\n");
						nrf_delay_ms(200);
				}
        NRF_LOG_INFO("SHT sensor probing successful\n");
        int32_t temperature, humidity;
        /* Measure temperature and relative humidity and store into variables
         * temperature, humidity (each output multiplied by 1000).
         */
        int8_t ret = sht4x_measure_blocking_read(&temperature, &humidity);
        if (ret == STATUS_OK)
				{
						 NRF_LOG_INFO("measured temperature: %3.2f degreeCelsius,"
													"measured humidity: %3.2f percentRH\n",temperature / 1000.0f, humidity / 1000.0f);
													 temp =temperature / 1000.0f;          
													 hum =humidity / 1000.0f;                     
                           temp_working_condition=1;
						NRF_LOG_INFO("temp=%f,hum=%f",temp,hum);
				} 
			  else 
			   {
						NRF_LOG_INFO("error reading measurement\n");
			   }

			 		// disable and uninitialize twi
		    nrf_drv_twi_disable(&i2c1_instance);
		    nrf_drv_twi_uninit(&i2c1_instance); 		
			  temp1=temp*100;
			  hum1=hum*100;	
	
			  ret_code_t error_code;
        error_code = app_timer_start(motion_sensor_timer_id, Motion_Data_Update, NULL);																		
        APP_ERROR_CHECK(error_code);			 			 			 
}

/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timers_init(void)
{
    // Initialize timer module.
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);

    // Create timers.

 
//		 // Timer to start/stop sending notification
//		err_code = app_timer_create(&m_adv_timer_id, APP_TIMER_MODE_REPEATED, adv_timeout_handler);
//    APP_ERROR_CHECK(err_code);
	
	   // advertising
	  err_code = app_timer_create(&m_repeated_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                repeated_timer_handler);
    APP_ERROR_CHECK(err_code);
    
	  // temp sensor
	  err_code = app_timer_create(&sensor_timer_id,
                                APP_TIMER_MODE_SINGLE_SHOT, 
                                sensor_timer_handler);
    APP_ERROR_CHECK(err_code);
	
	  // motion sensor
	  err_code = app_timer_create(&motion_sensor_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                motion_sensor_timer_handler);
    APP_ERROR_CHECK(err_code);
	
	
	
}


/**@brief Function for the GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
    ret_code_t              err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *)DEVICE_NAME,
                                          strlen((const char *)DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    /* YOUR_JOB: Use an appearance value matching the application's use case.
       err_code = sd_ble_gap_appearance_set(BLE_APPEARANCE_);
       APP_ERROR_CHECK(err_code); */

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for initializing the GATT module.
 */
static void gatt_init(void)
{
    ret_code_t err_code = nrf_ble_gatt_init(&m_gatt, NULL);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}

//static void adv_timeout_handler(void * p_context)
//{
//    UNUSED_PARAMETER(p_context);
//		send_notification();	
//}


//void send_notification(void)
//{

//		advertising_init();
//    ret_code_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);

//    APP_ERROR_CHECK(err_code);
//		

//}	



/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    ret_code_t         err_code;
    nrf_ble_qwr_init_t qwr_init = {0};

    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);

  
}


/**@brief Function for handling the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module which
 *          are passed to the application.
 *          @note All this function does is to disconnect. This could have been done by simply
 *                setting the disconnect_on_fail config parameter, but instead we use the event
 *                handler mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    ret_code_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling a Connection Parameters error.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    ret_code_t             err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for starting timers.
 */
static void application_timers_start(void)
{
    /* YOUR_JOB: Start your timers. below is an example of how to start a timer.
       ret_code_t err_code;
       err_code = app_timer_start(m_app_timer_id, TIMER_INTERVAL, NULL);
       APP_ERROR_CHECK(err_code); */			 
			 gap_params_init();
       gatt_init();
 
    services_init();
    conn_params_init();
   // peer_manager_init();
	   ret_code_t error_code;
     error_code = app_timer_start(sensor_timer_id, 
                                    ADV_DATA_UPDATE, 
                                    NULL);
     APP_ERROR_CHECK(error_code);
	
//			 ret_code_t err_code;
//       err_code = app_timer_start(m_adv_timer_id, ADV_INTERVAL, NULL);
//       APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
    ret_code_t err_code;

    err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_app_evt_wait(); //sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    ret_code_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            NRF_LOG_INFO("Fast advertising.");
            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_ADV_EVT_IDLE:
            sleep_mode_enter();
            break;

        default:
            break;
    }
}


/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    ret_code_t err_code = NRF_SUCCESS;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected.");
            // LED indication will be changed when advertising starts.
            break;

        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected.");
            err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            NRF_LOG_DEBUG("GATT Client Timeout.");
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            NRF_LOG_DEBUG("GATT Server Timeout.");
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for the Peer Manager initialization.
 */
//static void peer_manager_init(void)
//{
//    ble_gap_sec_params_t sec_param;
//    ret_code_t           err_code;

//    err_code = pm_init();
//    APP_ERROR_CHECK(err_code);

//    memset(&sec_param, 0, sizeof(ble_gap_sec_params_t));

//    // Security parameters to be used for all security procedures.
//    sec_param.bond           = SEC_PARAM_BOND;
//    sec_param.mitm           = SEC_PARAM_MITM;
//    sec_param.lesc           = SEC_PARAM_LESC;
//    sec_param.keypress       = SEC_PARAM_KEYPRESS;
//    sec_param.io_caps        = SEC_PARAM_IO_CAPABILITIES;
//    sec_param.oob            = SEC_PARAM_OOB;
//    sec_param.min_key_size   = SEC_PARAM_MIN_KEY_SIZE;
//    sec_param.max_key_size   = SEC_PARAM_MAX_KEY_SIZE;
//    sec_param.kdist_own.enc  = 1;
//    sec_param.kdist_own.id   = 1;
//    sec_param.kdist_peer.enc = 1;
//    sec_param.kdist_peer.id  = 1;

//    err_code = pm_sec_params_set(&sec_param);
//    APP_ERROR_CHECK(err_code);

//    err_code = pm_register(pm_evt_handler);
//    APP_ERROR_CHECK(err_code);
//}


/**@brief Clear bond information from persistent storage.
 */
static void delete_bonds(void)
{
    ret_code_t err_code;

    NRF_LOG_INFO("Erase bonds!");

    err_code = pm_peers_delete();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated when button is pressed.
 */
static void bsp_event_handler(bsp_event_t event)
{
    ret_code_t err_code;

    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break; // BSP_EVENT_SLEEP

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break; // BSP_EVENT_DISCONNECT

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break; // BSP_EVENT_KEY_0

        default:
            break;
    }
}

uint8_t adv_data_count = 0;
/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init()
{
    ret_code_t             err_code;
    ble_advertising_init_t init;
	  memset(&init, 0, sizeof(init));
		
	// Manufacturing data
		
	  ble_advdata_manuf_data_t 						    manuf_data; //Variable to hold manufacturer specific data
//    uint8_t my_data[]                       = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,adv_data_count};//20 bytes //Our data to advertise	
    manuf_data.company_identifier           = 0x0059; //Nordics company ID
//    manuf_data.data.p_data                  = my_data;
//    manuf_data.data.size                    = sizeof(my_data);
		 manuf_data.data.p_data                  = data_to_advertise;
    manuf_data.data.size                    = sizeof(data_to_advertise);	
    init.advdata.p_manuf_specific_data      = &manuf_data;

    init.advdata.name_type                  = BLE_ADVDATA_NO_NAME;//BLE_ADVDATA_SHORT_NAME;//BLE_ADVDATA_FULL_NAME;
	
    init.advdata.include_appearance         = false;//true;
    //init.advdata.flags                      = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    //init.advdata.uuids_complete.uuid_cnt    = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    //init.advdata.uuids_complete.p_uuids     = m_adv_uuids;
		 adv_data_count++;


    init.config.ble_adv_fast_enabled        = true;
    init.config.ble_adv_fast_interval       = APP_ADV_INTERVAL;
    init.config.ble_adv_fast_timeout        = APP_ADV_DURATION;

    init.evt_handler = on_adv_evt;

    err_code = ble_advertising_init(&m_advertising, &init);
    APP_ERROR_CHECK(err_code);

    ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}


/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
//static void buttons_leds_init(bool * p_erase_bonds)
//{
//    ret_code_t err_code;
//    bsp_event_t startup_event;

//    err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
//    APP_ERROR_CHECK(err_code);

//    err_code = bsp_btn_ble_init(NULL, &startup_event);
//    APP_ERROR_CHECK(err_code);

//    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
//}

/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init()
{
    ret_code_t err_code;
    bsp_event_t startup_event;

    err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);
	
    err_code = bsp_event_to_button_action_assign(BSP_BOARD_BUTTON_0, BSP_BUTTON_ACTION_RELEASE, BT0_LONG_PUSH); //add release action trigger on the button. call BSP_EVENT_KEY_2 event
    APP_ERROR_CHECK(err_code);      
}
/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    if (NRF_LOG_PROCESS() == false)
    {
        nrf_pwr_mgmt_run();
    }
}

/**@brief Function used for putting the chip into sleepmode.
   @detail When the Button pressed for more then 5sec then chip will reseset and code execute from main function
 */
void sleepmode()
{			
						bsp_wakeup_button_disable(BSP_BOARD_BUTTON_1);
						bsp_wakeup_button_disable(BSP_BOARD_BUTTON_2);
						bsp_wakeup_button_disable(BSP_BOARD_BUTTON_3);
						bsp_wakeup_button_enable(BSP_BOARD_BUTTON_0);
						bsp_indication_set(BSP_INDICATE_USER_STATE_0);
						nrf_delay_ms(50);
						bsp_indication_set(BSP_INDICATE_USER_STATE_OFF);
						NRF_LOG_INFO("Going to sleep.\r\n");
						nrf_delay_ms(100);			
						// Enter system OFF.
						NRF_POWER->SYSTEMOFF = 1;																
}
/**@brief Function for starting advertising.
 */
static void advertising_start()
{
        ret_code_t ret;
        ret = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
	      NRF_LOG_INFO("start advertising");
        APP_ERROR_CHECK(ret);
}


/**@brief Function for application main entry.
 */
int main(void)
{
   // bool erase_bonds;

    // Initialize.
    log_init();
    timers_init();
    buttons_leds_init();
	 //nrf_gpio_pin_clear(TEMP_VDD);
//	twi_init(); 
	//  LIS3DH_Init(); 
	//nrf_drv_twi_disable(&m_twi);          // disable twi
	  // nrf_drv_twi_uninit(&m_twi);  
	 //sensirion_i2c_init();
	  //nrf_drv_twi_disable(&i2c1_instance);
		  //  nrf_drv_twi_uninit(&i2c1_instance); 
	while(bsp_button_is_pressed(BSP_BOARD_BUTTON_0) == true)
    {
				if (button_tick >= 65534*100) //need to identify
			
					{
						
						button_tick = 0;
						bsp_indication_set(BSP_INDICATE_USER_STATE_0);
						NRF_LOG_INFO("BeforeFivesec"); 
						sleepFlag = 0;
						break;
				}
					button_tick++;
	  }	
	  while(sleepFlag)
		{     
				sleepmode();	
		}

	
    power_management_init();
    ble_stack_init();
    

    // Start execution.
    NRF_LOG_INFO("Template example started.");
    application_timers_start();



    // Enter main loop.
    for (;;)
    {
        idle_state_handle();
    }
}


/**
 * @}
 */
